@@ -12,14 +12,11 @@
 #endif
 
 #include "php.h"
-#include "ext/standard/php_smart_string.h"
+#include "ext/standard/php_smart_str.h"
 #include "ext/standard/php_array.h"
 #include "ext/standard/info.h"
 
 #include <rrd.h>
-#ifdef HAVE_RRDC_DISCONNECT
-#include <rrd_client.h>
-#endif
 
 #include "php_rrd.h"
 #include "rrd_graph.h"
@@ -39,7 +36,7 @@ PHP_FUNCTION(rrd_error)
 
 	if (!rrd_test_error()) RETURN_FALSE;
 
-	RETVAL_STRING(rrd_get_error());
+	RETVAL_STRING(rrd_get_error(), 1);
 	rrd_clear_error();
 }
 /* }}} */
@@ -50,7 +47,7 @@ Fetch data from RRD in requested resolution.
 PHP_FUNCTION(rrd_fetch)
 {
 	char *filename;
-	size_t filename_length;
+	int filename_length;
 	zval *zv_arr_options;
 	rrd_args *argv;
 	/* returned values if rrd_fetch doesn't fail */
@@ -60,14 +57,14 @@ PHP_FUNCTION(rrd_fetch)
 	char **ds_namv; /* list of data source names */
 	rrd_value_t *ds_data; /* all data from all sources */
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "pa", &filename,
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sa", &filename,
 		&filename_length, &zv_arr_options) == FAILURE) {
 		return;
 	}
 
-	if (php_check_open_basedir(filename)) RETURN_FALSE;
+	if (php_check_open_basedir(filename TSRMLS_CC)) RETURN_FALSE;
 
-	argv = rrd_args_init_by_phparray("fetch", filename, zv_arr_options);
+	argv = rrd_args_init_by_phparray("fetch", filename, zv_arr_options TSRMLS_CC);
 	if (!argv) {
 		zend_error(E_WARNING, "cannot allocate arguments options");
 		RETURN_FALSE;
@@ -76,13 +73,14 @@ PHP_FUNCTION(rrd_fetch)
 	if (rrd_test_error()) rrd_clear_error();
 
 	/* call rrd_fetch and test if fails */
-	if (rrd_fetch(argv->count - 1, &argv->args[1], &start, &end, &step, &ds_cnt,
+	if (rrd_fetch(argv->count-1, &argv->args[1], &start, &end, &step, &ds_cnt,
 		&ds_namv, &ds_data) == -1 ) {
+
 		rrd_args_free(argv);
 		RETURN_FALSE;
 	}
 
-	/* making return array */
+	/* making return array*/
 	array_init(return_value);
 	add_assoc_long(return_value, "start", start);
 	add_assoc_long(return_value, "end", end);
@@ -97,40 +95,49 @@ PHP_FUNCTION(rrd_fetch)
 		rrd_value_t *datap = ds_data;
 		uint timestamp, ds_counter;
 		/* final array for all data from all data sources */
-		zval zv_data_array;
+		zval *zv_data_array;
 
-		array_init(&zv_data_array);
+		MAKE_STD_ZVAL(zv_data_array)
+		array_init(zv_data_array);
 
 		/* add arrays for each data source, each array will be filled with
 		 * retrieved data for a particular data source
 		 */
 		for (ds_counter = 0; ds_counter < ds_cnt; ds_counter++) {
-			zval zv_ds_data_array;
-			array_init(&zv_ds_data_array);
+			zval *zv_ds_data_array;
+			MAKE_STD_ZVAL(zv_ds_data_array)
+			array_init(zv_ds_data_array);
 
-			add_assoc_zval(&zv_data_array, ds_namv[ds_counter], &zv_ds_data_array);
+			add_assoc_zval(zv_data_array, ds_namv[ds_counter], zv_ds_data_array);
 		}
 
 		for (timestamp = start + step; timestamp <= end; timestamp += step) {
 			/* try to find current data source result array in the
 			 * zv_data_array
 			 */
-			zend_hash_internal_pointer_reset(Z_ARRVAL(zv_data_array));
+			zend_hash_internal_pointer_reset(Z_ARRVAL_P(zv_data_array));
 			for (ds_counter = 0; ds_counter < ds_cnt; ds_counter++) {
 				/* pointer for one data source retrieved data */
-				zval *ds_data_array;
+				zval **ds_data_array;
 				/* value for key (timestamp) in data array */
-				char str_timestamp[11];
-				ZEND_LTOA((zend_ulong)timestamp, str_timestamp, sizeof(str_timestamp));
+				zval *zv_timestamp;
+
+				MAKE_STD_ZVAL(zv_timestamp);
+				ZVAL_LONG(zv_timestamp, timestamp);
+				convert_to_string(zv_timestamp);
 
 				/* gets pointer for data source result array */
-				ds_data_array = zend_hash_get_current_data(Z_ARRVAL(zv_data_array));
+				zend_hash_get_current_data(Z_ARRVAL_P(zv_data_array), (void**) &ds_data_array);
 
-				add_assoc_double(ds_data_array, str_timestamp, *(datap++));
-				zend_hash_move_forward(Z_ARRVAL(zv_data_array));
+				add_assoc_double(*ds_data_array, Z_STRVAL_P(zv_timestamp), *(datap++));
+
+				zend_hash_move_forward(Z_ARRVAL_P(zv_data_array));
+
+				zval_dtor(zv_timestamp);
+				efree(zv_timestamp);
 			}
 		}
-		add_assoc_zval(return_value, "data", &zv_data_array);
+		add_assoc_zval(return_value, "data", zv_data_array);
 
 		/* free data from rrd_fetch */
 		free(ds_data);
@@ -150,12 +157,12 @@ PHP_FUNCTION(rrd_fetch)
 PHP_FUNCTION(rrd_first)
 {
 	char *filename;
-	size_t filename_length;
+	int filename_length;
 	long rraindex = 0;
 	/* return value from rrd_first_r call */
 	time_t rrd_first_return_val;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "p|l", &filename,
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|l", &filename,
 		&filename_length, &rraindex) == FAILURE) {
 		return;
 	}
@@ -165,9 +172,7 @@ PHP_FUNCTION(rrd_first)
 		RETURN_FALSE;
 	}
 
-	if (php_check_open_basedir(filename)) {
-		RETURN_FALSE;
-	}
+	if (php_check_open_basedir(filename TSRMLS_CC)) RETURN_FALSE;
 
 	if (rrd_test_error()) rrd_clear_error();
 
@@ -186,18 +191,16 @@ PHP_FUNCTION(rrd_first)
 PHP_FUNCTION(rrd_last)
 {
 	char *filename;
-	size_t filename_length;
+	int filename_length;
 	/* return value from rrd_first_r call */
 	time_t rrd_last_return_val;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "p", &filename,
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename,
 		&filename_length) == FAILURE) {
 		return;
 	}
 
-	if (php_check_open_basedir(filename)) {
-		RETURN_FALSE;
-	}
+	if (php_check_open_basedir(filename TSRMLS_CC)) RETURN_FALSE;
 
 	if (rrd_test_error()) rrd_clear_error();
 
@@ -215,7 +218,7 @@ PHP_FUNCTION(rrd_last)
 PHP_FUNCTION(rrd_lastupdate)
 {
 	char *filename;
-	size_t filename_length;
+	int filename_length;
 	/* list of arguments for rrd_lastupdate call, it's more efficient then
 	 * usage of rrd_args, because there isn't array of arguments in parameters
 	 */
@@ -226,14 +229,12 @@ PHP_FUNCTION(rrd_lastupdate)
 	char **ds_namv;
 	char **last_ds;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "p", &filename,
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename,
 		&filename_length) == FAILURE) {
 		return;
 	}
 
-	if (php_check_open_basedir(filename)) {
-		RETURN_FALSE;
-	}
+	if (php_check_open_basedir(filename TSRMLS_CC)) RETURN_FALSE;
 
 	argv[0] = "dummy";
 	argv[1] = estrdup("lastupdate");
@@ -264,15 +265,16 @@ PHP_FUNCTION(rrd_lastupdate)
 		add_assoc_null(return_value, "ds_namv");
 	} else {
 		uint i;
-		zval zv_ds_namv_array;
-		array_init(&zv_ds_namv_array);
+		zval *zv_ds_namv_array;
+		MAKE_STD_ZVAL(zv_ds_namv_array);
+		array_init(zv_ds_namv_array);
 
 		for (i = 0; i < ds_cnt; i++) {
-			add_next_index_string(&zv_ds_namv_array, ds_namv[i]);
+			add_next_index_string(zv_ds_namv_array, ds_namv[i], 1);
 			free(ds_namv[i]);
 		}
 		free(ds_namv);
-		add_assoc_zval(return_value, "ds_navm", &zv_ds_namv_array);
+		add_assoc_zval(return_value, "ds_navm", zv_ds_namv_array);
 	}
 
 	/* "data" return array or null, if no available */
@@ -280,18 +282,19 @@ PHP_FUNCTION(rrd_lastupdate)
 		add_assoc_null(return_value, "data");
 	} else {
 		uint i;
-		zval zv_data_array;
-		array_init(&zv_data_array);
+		zval *zv_data_array;
+		MAKE_STD_ZVAL(zv_data_array);
+		array_init(zv_data_array);
 
 		/* simple array for "data" is enough, data source names and timestamps are
 		 * available under other return value keys
 		 */
 		for (i = 0; i < ds_cnt; i++) {
-			add_next_index_string(&zv_data_array, last_ds[i]);
+			add_next_index_string(zv_data_array, last_ds[i], 1);
 			free(last_ds[i]);
 		}
 		free(last_ds);
-		add_assoc_zval(return_value, "data", &zv_data_array);
+		add_assoc_zval(return_value, "data", zv_data_array);
 	}
 }
 
@@ -300,37 +303,35 @@ PHP_FUNCTION(rrd_lastupdate)
 PHP_FUNCTION(rrd_restore)
 {
 	char *xml_filename, *rrd_filename;
-	size_t xml_filename_length, rrd_filename_length;
+	int xml_filename_length, rrd_filename_length;
 	zval *zv_arr_options = NULL;
 	/* this is merge of options and rrd_filename. This is needed because
 	 * rrd_args_init_by_phparray allows only one filename as argument, so
 	 * rrd_filename mugst be part of array of arguments
 	 */
-	zval zv_options;
+	zval *zv_options;
 	rrd_args *argv;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "pp|a", &xml_filename,
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss|a", &xml_filename,
 		&xml_filename_length, &rrd_filename, &rrd_filename_length,
 		&zv_arr_options) == FAILURE) {
 		return;
 	}
 
-	if (php_check_open_basedir(xml_filename) ||
-		php_check_open_basedir(rrd_filename)) {
-		RETURN_FALSE;
-	}
+	if (php_check_open_basedir(xml_filename TSRMLS_CC)) RETURN_FALSE;
+	if (php_check_open_basedir(rrd_filename TSRMLS_CC)) RETURN_FALSE;
 
 	/* zv_options = merge rrd_filename and zv_arr_options content */
-	array_init(&zv_options);
-	add_next_index_string(&zv_options, rrd_filename);
+	MAKE_STD_ZVAL(zv_options);
+	array_init(zv_options);
+	add_next_index_string(zv_options, rrd_filename, 1);
 	if (zv_arr_options && Z_TYPE_P(zv_arr_options) == IS_ARRAY) {
-		php_array_merge(Z_ARRVAL(zv_options), Z_ARRVAL_P(zv_arr_options));
+		php_array_merge(Z_ARRVAL_P(zv_options), Z_ARRVAL_P(zv_arr_options), 0 TSRMLS_CC);
 	}
 
-	argv = rrd_args_init_by_phparray("restore", xml_filename, &zv_options);
+	argv = rrd_args_init_by_phparray("restore", xml_filename, zv_options TSRMLS_CC);
 	if (!argv) {
 		zend_error(E_WARNING, "cannot allocate arguments options");
-		zval_dtor(&zv_options);
 		RETURN_FALSE;
 	}
 
@@ -342,7 +343,7 @@ PHP_FUNCTION(rrd_restore)
 	} else {
 		RETVAL_TRUE;
 	}
-	zval_dtor(&zv_options);
+	zval_dtor(zv_options);
 	rrd_args_free(argv);
 }
 /* }}} */
@@ -352,11 +353,11 @@ PHP_FUNCTION(rrd_restore)
 PHP_FUNCTION(rrd_tune)
 {
 	char *filename;
-	size_t filename_length;
+	int filename_length;
 	zval *zv_arr_options;
 	rrd_args *argv;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "pa", &filename,
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sa", &filename,
 		&filename_length, &zv_arr_options) == FAILURE) {
 		return;
 	}
@@ -366,9 +367,9 @@ PHP_FUNCTION(rrd_tune)
 		RETURN_FALSE;
 	}
 
-	if (php_check_open_basedir(filename)) RETURN_FALSE;
+	if (php_check_open_basedir(filename TSRMLS_CC)) RETURN_FALSE;
 
-	argv = rrd_args_init_by_phparray("tune", filename, zv_arr_options);
+	argv = rrd_args_init_by_phparray("tune", filename, zv_arr_options TSRMLS_CC);
 	if (!argv) {
 		zend_error(E_WARNING, "cannot allocate arguments options");
 		RETURN_FALSE;
@@ -399,14 +400,14 @@ PHP_FUNCTION(rrd_xport)
 	ulong step, outvar_count;
 	char **legend_v;
 	rrd_value_t *data, *data_ptr;
-	zval zv_data;
+	zval *zv_data;
 	ulong outvar_index;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &zv_arr_options) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &zv_arr_options) == FAILURE) {
 		return;
 	}
 
-	argv = rrd_args_init_by_phparray("xport", "", zv_arr_options);
+	argv = rrd_args_init_by_phparray("xport", "", zv_arr_options TSRMLS_CC);
 	if (!argv) {
 		zend_error(E_WARNING, "cannot allocate arguments options");
 		RETURN_FALSE;
@@ -437,7 +438,8 @@ PHP_FUNCTION(rrd_xport)
 		return;
 	}
 
-	array_init(&zv_data);
+	MAKE_STD_ZVAL(zv_data);
+	array_init(zv_data);
 
 	for (outvar_index = 0; outvar_index < outvar_count; outvar_index++) {
 		/* array for a whole one output variable data, it contains indexes
@@ -449,26 +451,31 @@ PHP_FUNCTION(rrd_xport)
 		 *    ...
 		 *  ))
 		 */
-		zval zv_var_data, time_data;
-		array_init(&zv_var_data);
-		array_init(&time_data);
+		zval *zv_var_data, *time_data;
+		MAKE_STD_ZVAL(zv_var_data);
+		array_init(zv_var_data);
+		MAKE_STD_ZVAL(time_data);
+		array_init(time_data);
 
-		add_assoc_string(&zv_var_data, "legend", legend_v[outvar_index]);
+		add_assoc_string(zv_var_data, "legend", legend_v[outvar_index], 1);
 		free(legend_v[outvar_index]);
 
 		data_ptr = data + outvar_index;
-		for (time_index = start + step; time_index <= end; time_index += step) {
+		for (time_index = start+step; time_index <= end; time_index += step) {
 			/* value for key (timestamp) in data array */
-			char str_timestamp[11];
-			ZEND_LTOA((zend_ulong)time_index, str_timestamp, sizeof(str_timestamp));
+			zval *zv_timestamp;
+			MAKE_STD_ZVAL(zv_timestamp);
+			ZVAL_LONG(zv_timestamp, time_index);
+			convert_to_string(zv_timestamp);
 
-			add_assoc_double(&time_data, str_timestamp, *data_ptr);
+			add_assoc_double(time_data, Z_STRVAL_P(zv_timestamp), *data_ptr);
 			data_ptr += outvar_count;
+			zval_dtor(zv_timestamp);
 		}
-		add_assoc_zval(&zv_var_data, "data", &time_data);
-		add_next_index_zval(&zv_data, &zv_var_data);
+		add_assoc_zval(zv_var_data, "data", time_data);
+		add_next_index_zval(zv_data, zv_var_data);
 	}
-	add_assoc_zval(return_value, "data", &zv_data);
+	add_assoc_zval(return_value, "data", zv_data);
 	free(legend_v);
 	free(data);
 }
@@ -497,7 +504,7 @@ PHP_FUNCTION(rrd_version)
 		return;
 	}
 
-	RETVAL_STRING(rrd_strversion());
+	RETVAL_STRING(rrd_strversion(), 1);
 }
 
 /* {{{ arguments */
@@ -572,7 +579,7 @@ static zend_function_entry rrd_functions[] = {
 	PHP_FE(rrdc_disconnect, NULL)
 #endif
 	PHP_FE(rrd_version, NULL)
-	PHP_FE_END
+	{NULL, NULL, NULL}
 };
 /* }}} */
 
@@ -583,9 +590,9 @@ static zend_function_entry rrd_functions[] = {
 /* {{{ PHP_MINIT_FUNCTION */
 static PHP_MINIT_FUNCTION(rrd)
 {
-	rrd_graph_minit();
-	rrd_create_minit();
-	rrd_update_minit();
+	rrd_graph_minit(TSRMLS_C);
+	rrd_create_minit(TSRMLS_C);
+	rrd_update_minit(TSRMLS_C);
 	return SUCCESS;
 }
 /* }}} */
@@ -632,17 +639,16 @@ zend_module_entry rrd_module_entry = {
  * filename paremeter is optional.
  */
 rrd_args *rrd_args_init_by_phparray(const char *command_name, const char *filename,
-	const zval *options)
+	const zval *options TSRMLS_DC)
 {
 	uint i, option_count, args_counter = 2;
-	rrd_args *result;
 
 	if (Z_TYPE_P(options) != IS_ARRAY) return NULL;
 	option_count = zend_hash_num_elements(Z_ARRVAL_P(options));
 	if (!option_count) return NULL;
 	if (!strlen(command_name)) return NULL;
 
-	result = (rrd_args *)emalloc(sizeof(rrd_args));
+	rrd_args *result = (rrd_args *)emalloc(sizeof(rrd_args));
 	/* "dummy" + command_name + filename if presented */
 	result->count = option_count + (strlen(filename) ? 3 : 2);
 	result->args = (char **)safe_emalloc(result->count, sizeof(char *), 0);
@@ -655,18 +661,18 @@ rrd_args *rrd_args_init_by_phparray(const char *command_name, const char *filena
 	if (strlen(filename)) result->args[args_counter++] = estrdup(filename);
 
 	zend_hash_internal_pointer_reset(Z_ARRVAL_P(options));
-	for (i=0; i < option_count; i++) {
-		zval *item;
-		smart_string option = {0}; /* one argument option */
+	for (i=0; i<option_count; i++) {
+		zval **item;
+		smart_str option = {0}; /* one argument option */
 
 		/* force using strings as array items */
-		item = zend_hash_get_current_data(Z_ARRVAL_P(options));
-		if (Z_TYPE_P(item) != IS_STRING) convert_to_string(item);
-		smart_string_appendl(&option, Z_STRVAL_P(item), Z_STRLEN_P(item));
-		smart_string_0(&option);
+		zend_hash_get_current_data(Z_ARRVAL_P(options), (void**) &item);
+		if (Z_TYPE_PP(item) != IS_STRING) convert_to_string(*item);
+		smart_str_appendl(&option, Z_STRVAL_PP(item), Z_STRLEN_PP(item));
+		smart_str_0(&option);
 
 		result->args[args_counter++] = estrdup(option.c);
-		smart_string_free(&option);
+		smart_str_free(&option);
 
 		zend_hash_move_forward(Z_ARRVAL_P(options));
 	}
@@ -682,9 +688,8 @@ void rrd_args_free(rrd_args *args)
 	int i;
 	if (!args || !args->args) return;
 
-	for (i = 1; i < args->count; i++) {
+	for (i=1; i<args->count; i++)
 		efree(args->args[i]);
-	}
 
 	efree(args->args);
 	efree(args);
